#!/usr/bin/python
import os
import sys
import time
import serial
import struct
import subprocess
import csv, datetime

watchdogtime=0
#cont = True

def StopMBusHub():
    os.system("killall mbushub.elf")
    FoolWatchDog()
    os.system("killall mbushub.elf")


def FoolWatchDog():
    t = time.time()
    global watchdogtime
    if t > watchdogtime + 9:
        watchdogtime = t
        f = open("/tmp/mbushub.pid","wb")
        tb = struct.pack("<I",int(watchdogtime))
        f.write(tb)
        f.close()

def CalcBcc(array):
    cc=0
    for i in range(len(array)):
        cc = cc+array[i]
    return cc&0xFF

def PrintData(data):
    tstr = ""
    for j in range(len(data)):
        if j%16 == 0 and j != 0:
            tstr += "\n"
        tstr += "%02X "%data[j]
    print(tstr)

def PrintDataOnOneLine(data):
    tstr = ""
    for j in range(len(data)):
        tstr += "%02X "%data[j]
    print(tstr, end="")

# =============================================================================

def TransmitReceive(port, request, bytesToRead, timeout):
    port.timeout = timeout
    port.write(request)
    time.sleep(0.1)
    response = port.read(bytesToRead)
    return response

def CreateShortFrame(cfield, afield):
    req   = b"\x10"+struct.pack("BB",cfield, afield)
    req += struct.pack("B",CalcBcc(req[1:])) + b"\x16"
    return req

def CreateReqUd2(adress):
    toggle = 0x7B
    checkSum = (toggle + adress) & 0xFF
    req_ud2 = struct.pack("BBBBB", 0x10, toggle, adress, checkSum, 0x16)
    return req_ud2

def VerifyE5(resp): # Check for none ACK response.
    retval = ""
    if len(resp) == 0:
        retval = "No SND_NKE response"
        print(retval)
        return False
    elif len(resp) != 1 or resp[0] != 0xE5:
        retval = "Invalid SND_NKE_RESPONSE, %02X"%resp[0]
        print(retval)
        return False
    return True

def VerifyLongFrame(frame):
    if len(frame) == 0:
        print("No response received")
        return False
    if len(frame) < 21:  # Check number of received bytes.
        print("Too few bytes received")
        print(len(frame))
        return False
    if (frame[0] != 0x68 or frame[3] != 0x68): # Check frame characters.
        print("Incorrect frame characters")
        return False
    if (frame[1] != frame[2] or frame[1] != len(frame)-6): # Check length.
        print("Incorrect message length")
        return False
    if (CalcBcc(frame[4:len(frame)-2]) != frame[len(frame)-2]): # Check checksum.
        print("Incorrect checksum")
        return False
    return True

def MfctToStr(mfct):
    if mfct == 0xFFFF:
        return ""
    strng  = "%c"%(((mfct>>10)&0x1F)+64)
    strng += "%c"%(((mfct>>5)&0x1F)+64)
    strng += "%c"%((mfct&0x1F)+64) # Manufacturer number
    return strng

def DoSearch(serial_dev, ser_baudrate, startAddress, stopAddress, stop):
    s = None
    try:
        s = serial.Serial(serial_dev,
            #baudrate=2400,         # baudrate
            baudrate=ser_baudrate,
            bytesize=serial.EIGHTBITS,  # number of databits
            parity=serial.PARITY_EVEN,  # enable parity checking
            stopbits=serial.STOPBITS_ONE,   # number of stopbits
            timeout=1,         # set a timeout value, None for waiting forever
            xonxoff=0,          # enable software flow control
            rtscts=0            # enable RTS/CTS flow control
        )
    except:
        pass
    print("ser_baudrate", ser_baudrate)
    StopMBusHub()
    #cf=open("/tmp/meters_found.txt",'w', newline='')
    with open("/tmp/meters_found.txt", "w") as cf, open("/tmp/search_result.txt", "w") as test:
        writer = csv.writer(cf, delimiter=';', quotechar='\"', quoting=csv.QUOTE_MINIMAL)
        nodeaddresses =  [[0x4129,0x01,0x36], [0x1596,0x14,0x31]] # This should preferably be read from a configuration file.
        for addr in range(startAddress, stopAddress+1):
            #if not cont:
            #    break
            if stop.is_set():
                break
            FoolWatchDog()
            print("Sending REQ_UD2 on address: {}\t\t".format(addr), end="")
            test.write("REQ_UD2 address: {}\n".format(addr))
            test.flush()
            request = CreateShortFrame(0x40, addr)
            PrintDataOnOneLine(request)
            rec = TransmitReceive(s, request, 1024, 2.2)
            if len(rec) > 0: #Some answer is received
                meter = ['']*12
                request = CreateReqUd2(addr)
                PrintDataOnOneLine(request)
                rec = TransmitReceive(s, request, 1024, 2.2)
                if VerifyLongFrame(rec):
                    id,  mfct,  ver,  med = struct.unpack("<IHBB",rec[7:15])
                    meter[0] = "%08X"%id
                    meter[1] = "%04X"%mfct
                    meter[2] = "%02X"%ver
                    meter[3] = "%02X"%med
                    meter[4] = MfctToStr(mfct)
                    meter[6] = int(rec[5]) #Primary address
                    meter[7] = "20"
                    if [mfct, ver, med] in nodeaddresses:
                        meter[8] = "N"
                    else:
                        meter[8] = "W"
                    meter[9] = ""
                    meter[10] = ""
                    meter[11] = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                else:
                    meter[6] = addr # The meter has answered and we know the Addr is correct but REQ_UD2 did not work.
                writer.writerow(meter)
                cf.flush()
    #cf.close()
    s.close()

if __name__ == "__main__":
    startAddress = 1
    stopAddress = 251
    if len(sys.argv) > 1:
        startAddress = int(sys.argv[1])
    if len(sys.argv) > 2:
        stopAddress = int(sys.argv[2])
    serial_dev = "/dev/ttyS4"
    DoSearch(serial_dev, startAddress, stopAddress)
